
function histSeparation
      
plot_sep_hist = 0; % Put to 1 if separate phase and amplitude histograms shall be plotted
plot_hist = 0;

select = input('Select audio test sequences 1(speech+music)/2(speech+speech)/3(noisy speech+speech): ', 's');

% Speech + background music mixture
if (select == '1')
    [e1, fs, nbits1, readinfo1] = wavread('.\audio\rsm2_mA.wav');
    [e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rsm2_mB.wav');
% Speech + speech mixture
elseif (select == '2')
    [e1, fs, nbits1, readinfo1] = wavread('.\audio\rss_mA.wav');
    [e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rss_mB.wav');
% Noisy speech + speech mixture
elseif (select == '3')
    [e1, fs, nbits1, readinfo1] = wavread('.\audio\rssd_A.wav');
    [e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rssd_B.wav');
else
    [audio, fs, nbits2, readinfo2] = wavread('audio/dvorak1.wav');
    e1=audio(:,1);
    e2=audio(:,2);
end

select = input('Play sources before separation 1(left Mic)/2(right Mic): ', 's');

% Left source.
if (select == '1')
    sound(e1, fs)
    %         pause(15)
% Right source.
elseif (select == '2')
    sound(e2, fs)
    %         pause(15)
else
    fprintf('No sources selected.\n');
end

windowLength=1024;
overlap=0.5;
alpha=0.5;

windowed1 = windowSignal(e1, windowLength, overlap, alpha);
windowed2 = windowSignal(e2, windowLength, overlap, alpha);

diff = windowed1./windowed2;
amp = abs(diff);
amp = amp - 1./amp;
phase = atan2(imag(diff), real(diff));

dPhaseCorr = zeros(size(phase));
freq = 1:fs/2/(windowLength/2):fs;

for i=1:size(phase,1)
    dPhaseCorr(i,:) = phase(i,:)./freq;
end
dPhaseCorr = dPhaseCorr*windowLength;

% Clear the "wrong" amplitude samples
ampInd = abs(amp)>5;
amp(ampInd)=NaN;
dPhaseCorr(ampInd)=NaN;

dPhaseInd = abs(dPhaseCorr)>0.7;
amp(dPhaseInd)=NaN;
dPhaseCorr(dPhaseInd)=NaN;

% Plot the amplitude and phase histogram separately
if plot_sep_hist == 1
    figure;
    subplot(1,2,1);
    hist(amp(:),100);
    title('Amplitude Histogram');
    subplot(1,2,2);
    hist(dPhaseCorr(:),100)
    title('Phase Histogram');
end

ampSteps = 0:3/100:3;
phaseSteps = -0.4:0.8/100:0.4;

% Only plot histogram if wanted
if plot_hist == 1
    histmap = hist2(amp(:), dPhaseCorr(:), ampSteps, phaseSteps); 

    % Smooth the histogram
    filter = [0.2 1 1 1 0.2; 1 1 2 1 1; 1 2 3 2 1; 1 1 2 1 1; 0.2 1 1 1 0.2];
    filter = filter./sum(sum(filter));
    histSmoothed = conv2(filter, histmap);
    figure;
    surf(ampSteps, phaseSteps, histSmoothed(1:length(ampSteps),1:length(phaseSteps)));
    title('Smoothed Histogram');
    xlabel('Amplitude Ratio');
    ylabel('Phase difference');
    axis([min(ampSteps) max(ampSteps) min(phaseSteps) max(phaseSteps)])
end

% Use a gaussian mixture model
approxFit = ones(size(dPhaseCorr));
approxFit(isnan(dPhaseCorr))=NaN;
approxFit(dPhaseCorr(:)<0)=2;
approxFit = approxFit(~isnan(approxFit));

%X = [amp(:) atan(8*dPhaseCorr(:))];
X = [amp(:) dPhaseCorr(:)];
nbClusters = 2;
options = statset('Display','final');
gm = gmdistribution.fit(X,nbClusters,'Options',options,'Start',approxFit(:));
idx = cluster(gm,X);
cluster1 = (idx == 1);
cluster2 = (idx == 2);

cluster1d = double(reshape(cluster1,size(amp)));
cluster2d = double(reshape(cluster2,size(amp)));

filter = [1 1 1; 1 2 1; 1 1 1];
filter = filter./sum(sum(filter));
cluster1d = filter2(filter, cluster1d);
cluster2d = filter2(filter, cluster2d);

histmap = hist2(amp(cluster2), dPhaseCorr(cluster2), ampSteps, phaseSteps);
histSmoothed = conv2(filter, histmap);
% figure;
% surf(ampSteps, phaseSteps, histSmoothed(1:length(ampSteps),1:length(phaseSteps)));
% title('Smoothed Histogram after separation');
% xlabel('Amplitude Ratio');
% ylabel('Phase difference');
% axis([min(ampSteps) max(ampSteps) min(phaseSteps) max(phaseSteps)])


source1L = cluster1d.*windowed1;
source1R = cluster1d.*windowed2;

a1 = abs(source1L);
a1(:,windowLength/2+2:end) = fliplr(a1(:,2:windowLength/2));
p1 = atan2(imag(source1L), real(source1L));
p1(:,windowLength/2+2:end) = -fliplr(p1(:,2:windowLength/2));
a2 = abs(source1R);
a2(:,windowLength/2+2:end) = fliplr(a2(:,2:windowLength/2));
p2 = atan2(imag(source1R), real(source1R));
p2(:,windowLength/2+2:end) = -fliplr(p2(:,2:windowLength/2));

source1 = 0.5*(unwindowSignal(a1, p1, overlap, alpha)+unwindowSignal(a2,p2,overlap,alpha));


source2L = cluster2d.*windowed1;
source2R = cluster2d.*windowed2;

a1 = abs(source2L);
a1(:,windowLength/2+2:end) = fliplr(a1(:,2:windowLength/2));
p1 = atan2(imag(source2L), real(source2L));
p1(:,windowLength/2+2:end) = -fliplr(p1(:,2:windowLength/2));
a2 = abs(source2R);
a2(:,windowLength/2+2:end) = fliplr(a2(:,2:windowLength/2));
p2 = atan2(imag(source2R), real(source2R));
p2(:,windowLength/2+2:end) = -fliplr(p2(:,2:windowLength/2));

source2 = 0.5*(unwindowSignal(a1, p1, overlap, alpha)+unwindowSignal(a2,p2,overlap,alpha));

% Play separated sources.
select = input('Play sources after separation 1(left source)/2(right source): ', 's');

% Left source.
if (select == '1')
    sound(source1, fs)
    %         pause(15)
% Right source.
elseif (select == '2')
    sound(source2, fs)
    %         pause(15)
else
    fprintf('No sources selected.\n');
end
