plot_sep_hist = 0; % Put to 1 if separate phase and amplitude histograms shall be plotted

[e1, fs, nbits1, readinfo1] = wavread('.\audio\rsm2_mA.wav');
[e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rsm2_mB.wav');

% [e1, fs, nbits1, readinfo1] = wavread('.\audio\rss_mA.wav');
% [e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rss_mB.wav');

% [e1, fs, nbits1, readinfo1] = wavread('.\audio\rssd_A.wav');
% [e2, Fs2, nbits2, readinfo2] = wavread('.\audio\rssd_B.wav');

windowLength=512;
overlap=0.5;
alpha=0.5;

windowed1 = windowSignal(e1, windowLength, overlap, alpha);
windowed2 = windowSignal(e2, windowLength, overlap, alpha);

diff = windowed1./windowed2;
amp = abs(diff);
amp = amp - 1./amp;
phase = atan2(imag(diff), real(diff));

dPhaseCorr = zeros(size(phase));
freq = 1:fs/2/(windowLength/2):fs;

% size(freq)
% size(phase)

for i=1:size(phase,1)
    dPhaseCorr(i,:) = phase(i,:)./freq;
end
dPhaseCorr = dPhaseCorr*windowLength;

% Clear the "wrong" amplitude samples
ampInd = amp<3;
amp_clean = amp(ampInd);
dPhase_clean = dPhaseCorr(ampInd);

% Clear the "wrong" phase samples
phaseInd = abs(dPhase_clean)<0.5;
dPhase_clean = dPhase_clean(phaseInd);
amp_clean = amp_clean(phaseInd);

% Plot the amplitude and phase histogram separately
if plot_sep_hist == 1
    figure;
    subplot(1,2,1);
    hist(amp_clean,100);
    title('Amplitude Histogram');
    subplot(1,2,2);
    hist(dPhase_clean,100)
    title('Phase Histogram');
end

ampSteps = 0:3/100:3;
phaseSteps = -0.4:0.8/100:0.4;

histmap = hist2(amp_clean(:), dPhase_clean(:), ampSteps, phaseSteps); 
% figure;
% surf(ampSteps, phaseSteps, histmap);
% title('Raw Histogram');
% xlabel('Amplitude Ratio');
% ylabel('Phase difference');
% axis([min(ampSteps) max(ampSteps) min(phaseSteps) max(phaseSteps)])

% Smooth the histogram
filter = [0.2 1 1 1 0.2; 1 1 2 1 1; 1 2 3 2 1; 1 1 2 1 1; 0.2 1 1 1 0.2];
filter = filter./sum(sum(filter));
histSmoothed = conv2(filter, histmap);
figure;
surf(ampSteps, phaseSteps, histSmoothed(1:length(ampSteps),1:length(phaseSteps)));
title('Smoothed Histogram');
xlabel('Amplitude Ratio');
ylabel('Phase difference');
axis([min(ampSteps) max(ampSteps) min(phaseSteps) max(phaseSteps)])

idx = kmeans([amp(:) dPhaseCorr(:)], 2);

histmap = hist2(amp(idx==2), dPhaseCorr(idx==2), ampSteps, phaseSteps);
figure; surf(histmap);
filter = [0.2 1 1 1 0.2; 1 1 2 1 1; 1 2 3 2 1; 1 1 2 1 1; 0.2 1 1 1 0.2];
filter = filter./sum(sum(filter));
histSmoothed = conv2(filter, histmap);
figure;
surf(ampSteps, phaseSteps, histSmoothed(1:length(ampSteps),1:length(phaseSteps)));
title('Smoothed Histogram after separation');
xlabel('Amplitude Ratio');
ylabel('Phase difference');
axis([min(ampSteps) max(ampSteps) min(phaseSteps) max(phaseSteps)])

% Ask for the separation line of the sources
display('Please choose the line separating the two sources');
[ampSep phaseSep] = ginput(2);

% Find the line connecting the two points
slope = (phaseSep(2)-phaseSep(1))/(ampSep(2)-ampSep(1));
cross = phaseSep(1) - slope*ampSep(1);

% Do the classification
mask = zeros(size(windowed1));
for i=1:size(windowed1,1)
    for j=1:size(windowed1,2)
        if dPhaseCorr(i,j)>slope*amp(i,j)+cross
            mask(i,j) = 1;
        end
    end
end

source1L = mask.*windowed1;
%source1R = mask.*windowed2;

%source1R(:,windowLength/2+2:end) = fliplr(source1R(:,2:windowLength/2));
a = abs(source1L);
a(:,windowLength/2+2:end) = fliplr(a(:,2:windowLength/2));
p = atan2(imag(source1L), real(source1L));
p(:,windowLength/2+2:end) = -fliplr(p(:,2:windowLength/2));

source1LTime = unwindowSignal(a, p, overlap, alpha);