% Perform the inverse fourier transform of a windowed signal. The parameters
% amplitude and phase are the amplitude and phases of the fft of the 
% windowed signal and synWin is the synthesis window to be applied.
% The function returns the windowed inverse fourier transform.

function xw = iwfft (amplitude, phase, synWin)

    x = amplitude .* exp(1j*phase); % Compose the complex frequency representation
    xw = synWin' .* ifft(x); % Take the ifft and window it with the synthesis window
    maximag = max(abs(imag(xw))); % Search for the largest imaginary value
    
    % Plot an error if the ifft contains significant complex components
    if (maximag > 1e-11)
        display(['Max Imag: ' num2str(maximag)]); 
        error('Inverse DFT is not purely real');
    else
        xw = real(xw);
    end

return