
%--------------------------------------------------------------------
%
% [mask1 mask2] = maskGen1(dPhase, dLevel, lTh, pTh)
%
% Arguments:
% ---------------
% dPhase               phase difference matrix
% dLevel               level difference matrix
% lTh                  threshold for level separation
% pTh                  threshold for phase separation
%
% Returns:
% ---------------
% mask1                mask for source1
% mask2                mask for source2
%--------------------------------------------------------------------

function [mask1, mask2] = maskGen1(dPhase, dLevel, lTh, pTh)

[m, n] = size(dPhase);

levelMask1 = zeros(m, (n-1) * 2);
phaseMask1 = zeros(m, (n-1) * 2);

levelMask2 = zeros(m, (n-1) * 2);
phaseMask2 = zeros(m, (n-1) * 2);

% Mask generation.
for i = 1 : m
    for j = 1 : n
        
        % Amplitude mask generation.
        
        % Thresholding.
        if (dLevel(i, j) > lTh)
            levelMask1(i, j) = 1.0;
        elseif (dLevel(i, j) < -lTh)
            levelMask2(i, j) = 1.0;
        % Linear weighting.
        elseif (dLevel(i, j) >= 0 && dLevel(i, j) < lTh)
            levelMask2(i, j) = 1e-2;
            levelMask1(i, j) = 1 / lTh * dLevel(i, j) + 1e-2;
        elseif (dLevel(i, j) >= -lTh && dLevel(i, j) < 0)
            levelMask1(i, j) = 1e-2;
            levelMask2(i, j) = 1 / lTh * abs(dLevel(i, j)) + 1e-2;
        % Exponential weighting.
%         elseif (dLevel(i, j) >= 0 && dLevel(i, j) < lTh)
%             levelMask2(i, j) = 1e-2;
%             levelMask1(i, j) = exp(2 * (dLevel(i, j) - lTh));
%         elseif (dLevel(i, j) >= -lTh && dLevel(i, j) < 0)
%             levelMask1(i, j) = 1e-2;
%             levelMask2(i, j) = exp(-2 * (dLevel(i, j) + lTh));
        % Residue scaling.
        else
            levelMask1(i, j) = 1e-2;
            levelMask2(i, j) = 1e-2;
        end
        
        % Phase mask generation.
        
        if (dPhase(i, j) > pTh)
            phaseMask2(i, j) = 1.0;
        elseif (dPhase(i, j) < -pTh)
            phaseMask1(i, j) = 1.0;
        else
            phaseMask1(i, j) = 1e-2;
            phaseMask2(i, j) = 1e-2;
        end 
    end
end

% Mask generation.

%mask1 = levelMask1;
%mask1 = levelMask1 .* phaseMask1;
mask1 = (0.9 * levelMask1 + 0.1 * phaseMask1);

%mask2 = levelMask2;
%mask2 = levelMask2 .* phaseMask2;
mask2 = (0.9 * levelMask2 + 0.1 * phaseMask2);

% Flip to the other half for symmertry.
mask1(:,n+1:end) = fliplr(mask1(:,2:n-1));
mask2(:,n+1:end) = fliplr(mask2(:,2:n-1));























