
%--------------------------------------------------------------------
% Source separation with time-frequency masking
%--------------------------------------------------------------------

function tfSeparation

select = input('Select audio test sequences 1(speech+music)/2(speech+speech)/3(noisy speech+speech): ', 's');

% Speech + background music mixture
if (select == '1')
    leftChannelFile = '.\audio\rsm2_mA.wav';
    rightChannelFile = '.\audio\rsm2_mB.wav';
    % Speech + speech mixture
elseif (select == '2')
    leftChannelFile = '.\audio/rss_mA.wav';
    rightChannelFile = '.\audio/rss_mB.wav';
    % Noisy speech + speech mixture
elseif (select == '3')
    leftChannelFile = '.\audio/rssd_A.wav';
    rightChannelFile = '.\audio/rssd_B.wav';
    % Default speech + speech mixture (classical cocktail-party processing)
else
    leftChannelFile = '.\audio/rss_mA.wav';
    rightChannelFile = '.\audio/rss_mB.wav';
end

% Read the sound
[leftChannel,fs] = wavread(leftChannelFile);
rightChannel = wavread(rightChannelFile);

select = input('Play sources before separation 1(left Mic)/2(right Mic): ', 's');

% Left source.
if (select == '1')
    sound(leftChannel, fs)
    %         pause(15)
    % Right source.
elseif (select == '2')
    sound(rightChannel, fs)
    %         pause(15)
else
    fprintf('No sources selected.\n');
end

% Perform the windowed FFT
% Define additional parameters
windowLength = 1024;
overlap = 0.5;
alpha = 0.5;

% Perform the windowing
leftWindowed = windowSignal(leftChannel, windowLength, overlap, alpha);
leftAmplitude = abs(leftWindowed);
leftPhase = atan2(imag(leftWindowed), real(leftWindowed));

rightWindowed = windowSignal(rightChannel, windowLength, overlap, alpha);
rightAmplitude = abs(rightWindowed);
rightPhase = atan2(imag(rightWindowed), real(rightWindowed));

dPhase = leftPhase(:,1:windowLength/2 + 1)-rightPhase(:,1:windowLength/2 + 1);
% figure;
% surf(dPhase);
% zlabel('phase difference (rad)');

dLevel = 10*log10(abs(leftAmplitude(:,1:windowLength/2 + 1)./rightAmplitude(:,1:windowLength/2 + 1)));
% figure;
% surf(dLevel);
% zlabel('amplitude level difference (dB)');

% Generate mask.
[mask1, mask2] = maskGen1(dPhase, dLevel, 5.0, 1.0 * pi);

% Source separation.
leftAmplitude1 = leftAmplitude .* mask1;
leftPhase1 = leftPhase;
%       rightAmplitude1 = rightAmplitude .* mask1;
%       rightPhase1 = rightPhase;

%       leftAmplitude2 = leftAmplitude .* mask2;
%       leftPhase2 = leftPhase;
rightAmplitude2 = rightAmplitude .* mask2;
rightPhase2 = rightPhase;

% Generate separated sources.
leftSource1 = unwindowSignal(leftAmplitude1, leftPhase1, overlap, alpha)';
rightSource2 = unwindowSignal(rightAmplitude2, rightPhase2, overlap, alpha)';

% Play separated sources.
select = input('Play sources after separation 1(left source)/2(right source): ', 's');

% Left source.
if (select == '1')
    sound(leftSource1, fs)
    %         pause(15)
    % Right source.
elseif (select == '2')
    sound(rightSource2, fs)
    %         pause(15)
else
    fprintf('No sources selected.\n');
end




