% Unbuffers additively a signal using the specified overlap. The first overlap
% number samples of a column of signal overlap the last overlap number samples 
% of the previous column.

% For the final output the first overlap samples (corresponding to the zeros
% inserted by the buffer function are removed.

% Total length of the signal is Lf (first column) + (Nf-1)*(Lf-P) (remaining
% columns)

function y = unBuffer (signal, overlap)

    [windowLength, Nf] = size(signal);
    nOverlap = windowLength * overlap;
    Nadv = windowLength - nOverlap;
    Ny = windowLength + (Nf-1)*Nadv;
    y = zeros(Ny, 1);

    for k = 1:Nf
      ist = (k-1)*Nadv + 1;
      y(ist:ist+windowLength-1) = y(ist:ist+windowLength-1) + signal(:,k);
    end

    y(1:nOverlap) = [];

return