% Windows the signal in signal into frames of length windowLength and
% overlap overlap (fraction of entire frame length) and parameter alpha for
% the windowing function (modified Hann window)
% Returns the windowed signal where each line corresponds to one frame

% Notes:
% - windowLength should be a multiple of a power of two in order to speed 
%   up the fft computation
% 

function windowed = windowSignal(signal, windowLength, overlap, alpha)

% Analysis / synthesis parameters
nOverlap = windowLength * overlap;

% Generate the analysis window
wa = FracHannWin(windowLength, alpha);

% Plot the analysis/synthesis windows
% plot(0:windowLength-1, wa, 0:windowLength-1, ws);
% axis([0, windowLength, 0, 1.2]);

% Arrange the data frame by frame
% windowLength-NAdv zeros are inserted in the first frame to prevent boundary
% effects
P = windowLength - nOverlap; 
xbq = buffer(signal, windowLength, P);
[foo, NbFrames] = size(xbq);

% Allocate storage for the processing
windowed = NaN(NbFrames, windowLength);
XA = NaN(NbFrames, windowLength);
XP = NaN(NbFrames, windowLength);

% Main loop over frames to take the wfft
for n = 1:NbFrames
    % Analysis window / FFT / Amplitude-Phase
    [windowed(n,:), XA(n,:), XP(n,:)] = wfft(xbq(:,n), wa);
end